# -*- coding: utf-8 -*-
""" Viewing components for WAeUP objects.
"""
import copy
import csv
import grok
import os
import re
import sys
import time
import re
import smtplib
from email.mime.text import MIMEText
from hurry import yui
from hurry.workflow.interfaces import NoTransitionAvailableError
from zope import schema
from waeup.sirp.browser.resources import warning
try:
    from zope.authentication.interfaces import (
        IAuthentication, IUnauthenticatedPrincipal, ILogout)
except ImportError:
    # BBB
    from zope.app.security.interfaces import (
        IAuthentication, IUnauthenticatedPrincipal, ILogout)
from zope.component import (
    getUtility, createObject,getAllUtilitiesRegisteredFor)
from zope.component.interfaces import Invalid
from zope.exceptions import DuplicationError
from zope.session.interfaces import ISession

from waeup.sirp.browser import (
    WAeUPPage, WAeUPEditFormPage, WAeUPAddFormPage,
    WAeUPDisplayFormPage, NullValidator)
from waeup.sirp.browser.interfaces import (
    IUniversity, IFacultyContainer, IFaculty, IDepartment, ICourse,
    ICertificate, ICertificateCourse)
from waeup.sirp.catalog import search_context
from waeup.sirp.interfaces import(
    IWAeUPObject, IUserContainer, IUserAccount, IDataCenter,
    IWAeUPXMLImporter, IWAeUPXMLExporter, IBatchProcessor)
from waeup.sirp.university.catalog import search
from waeup.sirp.utils.helpers import getName
from waeup.sirp.widgets.interfaces import ITableProvider

grok.context(IWAeUPObject)
grok.templatedir('templates')

#
# Login/logout pages...
#

class LoginPage(WAeUPPage):
    """A login page, available for all objects.
    """
    grok.name('login')
    grok.context(IWAeUPObject)
    grok.require('waeup.Public')

    title = u'Login'
    camefrom = None

    def update(self, SUBMIT=None, camefrom=None):
        self.camefrom = camefrom
        if SUBMIT is not None:
            if self.request.principal.id != 'zope.anybody':
                self.flash('You logged in.')
                if not self.camefrom:
                    # User might have entered the URL directly. Let's beam
                    # him back to our context.
                    self.redirect(self.url(self.context))
                    return
                self.redirect(self.camefrom)
                return
            self.flash('You entered wrong credentials!')


class LoginStaffPage(LoginPage):
    """A login page for staff members.
    """
    grok.name('loginstaff')
    grok.template('loginstaffpage')
    grok.require('waeup.Public')
    title = u'Staff Login'


class LoginStudentPage(LoginPage):
    """A login page for students.
    """
    grok.name('loginstudent')
    grok.template('loginstudentpage')
    grok.require('waeup.Public')
    title = u'Student Login'

class LogoutPage(WAeUPPage):
    """A logout page. Calling this page will log the current user out.
    """
    grok.context(IWAeUPObject)
    grok.require('waeup.Public')
    grok.name('logout')
    
    def update(self):
        if not IUnauthenticatedPrincipal.providedBy(self.request.principal):
            auth = getUtility(IAuthentication)
            ILogout(auth).logout(self.request)
            self.flash("You have been logged out. Thanks for using WAeUP SIRP!")
        self.redirect(self.url(self.context))


#
# University related pages...
#

class UniversityPage(WAeUPPage):
    """ The main university page.
    """
    grok.require('waeup.Public')
    grok.name('index')
    grok.context(IUniversity)
    title = u'Welcome to SIRP'
    pnav = 0

class Manage(WAeUPEditFormPage):
    """Manage the basic properties of a `University` instance.
    """
    form_fields = grok.AutoFields(IUniversity)
    grok.context(IUniversity)
    grok.require('waeup.manageUniversity')
    title = u'Edit portal settings'
    pnav = 0

    @grok.action('Save')
    def save(self, **data):
        self.applyData(self.context, **data)
        self.flash('Settings have been saved.')
        return
    
    @grok.action('Save and return')
    def saveAndReturn(self, **data):
        self.applyData(self.context, **data)
        self.redirect(self.url(self.context))
        self.flash('Settings have been saved.')
        return
    @grok.action('Cancel', validator=NullValidator)
    def cancel(self, **data):
        self.flash('Action cancelled.')
        self.redirect(self.url(self.context))
        return

    @grok.action('Update plugins', validator=NullValidator)
    def updatePlugins(self, **data):
        self.context.updatePlugins()
        self.flash('Plugins were updated. See log file for details.')
        self.redirect(self.url(self.context))
        return
        
class RemoveFormPage(WAeUPPage):
    grok.require('waeup.manageUniversity')
    grok.name('remove')
    grok.template('removesubobjectspage')
    pnav = 1

    def update(self):
        form = self.request.form
        if 'CANCEL' in form.keys():
            self.redirect(self.url(self.context))
        if not 'REMOVE' in form.keys():
            return
        child_id = form['val_id']
        if not isinstance(child_id, list):
            child_id = [child_id]
        deleted = []
        for id in child_id:
            try:
                del self.context[id]
                deleted.append(id)
            except:
                self.flash('Could not delete %s: %s: %s' % (
                        id, sys.exc_info()[0], sys.exc_info()[1]))
        if len(deleted):
            self.flash('Successfully removed: %s' % ', '.join(deleted))
        # We have to redirect to let flash messages appear immediately...
        self.redirect(self.url())
        return

class AdministrationPage(WAeUPPage):
    """ The administration overview page.
    """
    grok.name('administration')
    grok.context(IUniversity)
    grok.require('waeup.manageUniversity')
    title = u'Administration'
    pnav = 0

class RSS20Feed(grok.View):
    """An RSS 2.0 feed.
    """
    grok.name('feed.rss')
    grok.context(IUniversity)
    grok.require('waeup.Public')
    grok.template('universityrss20feed')

    name = 'General news feed'
    description = 'waeup.sirp now supports RSS 2.0 feeds :-)'
    language = None
    date = None
    buildDate = None
    editor = None
    webmaster = None

    @property
    def title(self):
        return getattr(grok.getSite(), 'name', u'Sample University')

    @property
    def contexttitle(self):
        return self.name

    @property
    def link(self):
        return self.url(grok.getSite())

    def update(self):
        self.response.setHeader('Content-Type', 'text/xml; charset=UTF-8')

    def entries(self):
        return ()
    
#
# User container pages...
#
    
class UserContainerPage(WAeUPPage):
    """Overview page for all local users.
    """
    grok.require('waeup.manageUsers')
    grok.context(IUserContainer)
    grok.name('index')
    title = 'Portal Users'
    
    def update(self, userid=None, adduser=None, edit=None, delete=None):
        if edit is not None and userid is not None:
            self.redirect(self.url(userid))
        if delete is not None and userid is not None:
            self.context.delUser(userid)
            self.flash('User %s successfully deleted.' % userid)

class AddUserFormPage(WAeUPAddFormPage):
    grok.require('waeup.manageUsers')
    grok.context(IUserContainer)
    grok.name('add')
    form_fields = grok.AutoFields(IUserAccount)
    label = ''
    title = 'Add user'

    @grok.action('Add user')
    def addUser(self, **data):
        name = data['name']
        title = data['title']
        description = data['description']
        password = data['password']
        roles = data['roles']
        try:
            self.context.addUser(name, password, title=title,
                                 description=description, roles=roles)
        except KeyError:
            self.status = Invalid('The userid chosen already exists '
                                  'in the database')
            return
        except DuplicationError:
            self.status = Invalid('The userid chosen already exists '
                                  'in the database')
            return
        self.redirect(self.url(self.context))

class UserEditFormPage(WAeUPEditFormPage):
    """Edit a user account.
    """
    grok.context(IUserAccount)
    grok.name('index')
    
    form_fields = grok.AutoFields(IUserAccount)
    grok.require('waeup.manageUsers')

    title = "Edit user"

    @grok.action('Save')
    def save(self, **data):
        self.applyData(self.context, **data)
        self.flash('User settings have been saved.')
        return
    
    @grok.action('Save and return')
    def saveAndReturn(self, **data):
        self.applyData(self.context, **data)
        self.flash('User settings have been saved.')
        self.redirect(self.url(self.context.__parent__))
        return

    @grok.action('Cancel', validator=NullValidator)
    def cancel(self, **data):
        self.redirect(self.url(self.context.__parent__))
        return

#
# Search pages...
#
class SearchStudentPage(WAeUPPage):
    grok.context(IUniversity)
    grok.name('searchstudent')
    grok.template('searchstudentpage')
    grok.require('waeup.View')
    title = u"Student Search"
    pnav = 2

    def update(self, query=None):
        self.search_result = []
        if query is not None:
            self.search_result = search_context(query)
            print "RESULT: ", list(self.search_result)


class SearchPage(WAeUPPage):
    grok.context(IUniversity)
    grok.name('search')
    grok.template('searchpage')
    grok.require('waeup.View')
    title = u"Site Search"
    pnav = 2

    def update(self, *args, **kw):
        form = self.request.form
        self.hitlist = []
        self.query = ''
        if not 'query' in form:
            return
        query = form['query']
        self.query = query
        self.hitlist = search(query=self.query, view=self)
        return
        
#
# Contact forms...
#
class ContactAdminForm(WAeUPPage):
    grok.context(IUniversity)
    grok.name('contactadmin')
    grok.template('contactadminform')
    grok.require('waeup.Public')
    pnav = 2
    
    def title(self):
        """Return True if the calling user is authenticated.
        """
        userid = self.request.principal.id
        if userid != 'zope.anybody':
            tt = u'Contact' 
        else:
            tt = u'Enquiries'         
        return tt     
        
    def update(self, *args, **kw):
        form = self.request.form
        if not ('fullname' in form and 'email' in form and 'descr' in form):
            return
        self.fullname = fullname = form['fullname']
        email = form['email']
        self.descr = descr = form['descr']
        regno =form['regno']
        if not (fullname and email and descr):
            self.flash('Error: All fields must be filled!')
            return        
        if not re.match("^[a-zA-Z0-9._%-]+@[a-zA-Z0-9._%-]+.[a-zA-Z]{2,6}$",
                        email):
            self.flash('Error: %s is not a valid email address!' % email)
            return
        if regno == 'zope.anybody':
            regno = 'Anonymous User'
        text = """Fullname: %s
Member ID: %s
Description: %s
"""
        msg = MIMEText(text % (fullname,regno,descr))
        msg['From'] = '%s <%s>' % (fullname,email)
        
        # These parameters should be part of the portal configuration.
        msg['To'] = 'contact@waeup.org'
        msg['Subject'] = 'WAeUP Contact'
        #server = smtplib.SMTP('smtp_server')
        # Also connection details (username, passwords, host, port,
        # etc.) should be settable somewhere.
        # server.login('user_name','secret_password')
        server = smtplib.SMTP('localhost')
        server.sendmail(email,'contact@waeup.org',msg.as_string())
        
        server.quit()
        self.flash('Your message has been sent.')
        return 

#
# Datacenter pages...
#

class DatacenterPage(WAeUPPage):
    grok.context(IDataCenter)
    grok.name('index')
    grok.require('waeup.manageUniversity')
    title = u'Data Center'
    pnav = 0

class DatacenterUploadPage(WAeUPPage):
    grok.context(IDataCenter)
    grok.name('upload')
    grok.require('waeup.manageUniversity')
    title = u'Data Center Upload'
    pnav = 0
    
    def update(self, uploadfile=None, CANCEL=None, SUBMIT=None):
        if CANCEL is not None:
            self.redirect(self.url(self.context))
            return
        if not uploadfile:
            return
        try:
            filename = uploadfile.filename
            target = os.path.join(self.context.storage,
                                  self.getNormalizedFileName(filename))
            open(target, 'wb').write(uploadfile.read())
            logger = self.context.logger
            logger.info('%s: Uploaded file "%s"' % (
                self.request.principal.id, target))
        except IOError:
            self.flash('Error while uploading file. Please retry.')
            self.flash('I/O error: %s' % sys.exc_info()[1])
            return
        self.redirect(self.url(self.context))

    def getNormalizedFileName(self, filename):
        """Build sane filename.

        An uploaded file foo.csv will be stored as foo_USERNAME.csv
        where username is the principal id of the currently logged in
        user.

        Spaces in filename are replaced by underscore.
        """
        username = self.request.principal.id
        filename = filename.replace(' ', '_')
        # Only accept typical filname chars...
        filtered_username = ''.join(re.findall('[a-zA-Z0-9_\.\-]', username))
        base, ext = os.path.splitext(filename)
        return '%s_%s%s' % (base, filtered_username, ext.lower())

class DatacenterImportStep1(WAeUPPage):
    """Manual import step 1: choose file
    """
    grok.context(IDataCenter)
    grok.name('import1')
    grok.template('datacenterimport1page')
    grok.require('waeup.manageUniversity')
    title = u'Process CSV file'
    pnav = 0

    def getFiles(self):
        files = self.context.getFiles(sort='date')
        for file in files:
            name = file.name
            if not name.endswith('.csv') and not name.endswith('.pending'):
                continue
            yield file
    
    def update(self, filename=None, select=None, cancel=None):
        if cancel is not None:
            self.flash('Import aborted')
            self.redirect(self.url(self.context))
            return
        if select is not None:
            # A filename was selected
            session = ISession(self.request)['waeup.sirp']
            session['import_filename'] = select
            self.redirect(self.url(self.context, '@@import2'))

class DatacenterImportStep2(WAeUPPage):
    """Manual import step 2: choose importer
    """
    grok.context(IDataCenter)
    grok.name('import2')
    grok.template('datacenterimport2page')
    grok.require('waeup.manageUniversity')
    title = u'Process CSV file'
    pnav = 0

    filename = None
    mode = 'create'
    importer = None
    mode_locked = False

    def getPreviewHeader(self):
        """Get the header fields of attached CSV file.
        """
        reader = csv.reader(open(self.fullpath, 'rb'))
        return reader.next()
    
    def getPreviewBody(self):
        """Get the first 5 rows of attached CSV file.
        """
        result = []
        num = 0
        for row in self.reader:
            if num > 4:
                break
            num += 1
            row = row.items()
            # Sort fields in headerfield order
            row = sorted(row, key=lambda k: self.reader.fieldnames.index(k[0]))
            row = [x[1] for x in row]
            result.append(row)
        result.append(len(result[0]) * ['...'])
        return result

    def getImporters(self):
        importers = getAllUtilitiesRegisteredFor(IBatchProcessor)
        importers = [
            dict(title=x.name, name=x.util_name) for x in importers]
        return importers

    def getModeFromFilename(self, filename):
        """Lookup filename or path and return included mode name or None.
        """
        if not filename.endswith('.pending.csv'):
            return None
        base = os.path.basename(filename)
        parts = base.rsplit('.', 3)
        if len(parts) != 4:
            return None
        if parts[1] not in ['create', 'update', 'remove']:
            return None
        return parts[1]

    def update(self, mode=None, importer=None,
               back1=None, cancel=None, proceed=None):
        session = ISession(self.request)['waeup.sirp']
        self.filename = session.get('import_filename', None)
        
        if self.filename is None or back1 is not None:
            self.redirect(self.url(self.context, '@@import1'))
            return
        if cancel is not None:
            self.flash('Import aborted')
            self.redirect(self.url(self.context))
            return
        self.mode = mode or session.get('import_mode', self.mode)
        filename_mode = self.getModeFromFilename(self.filename)
        if filename_mode is not None:
            self.mode = filename_mode
            self.mode_locked = True
        self.importer = importer or session.get('import_importer', None)
        session['import_mode'] = self.mode
        session['import_importer'] = self.importer
        if proceed is not None:
            self.redirect(self.url(self.context, '@@import3'))
            return
        self.fullpath = os.path.join(self.context.storage, self.filename)
        self.reader = csv.DictReader(open(self.fullpath, 'rb'))

class DatacenterImportStep3(WAeUPPage):
    """Manual import step 3: modify header
    """
    grok.context(IDataCenter)
    grok.name('import3')
    grok.template('datacenterimport3page')
    grok.require('waeup.manageUniversity')
    title = u'Process CSV file'
    pnav = 0

    filename = None
    mode = None
    importername = None
    
    @property
    def nextstep(self):
        return self.url(self.context, '@@import4')

    def getPreviewHeader(self):
        """Get the header fields of attached CSV file.
        """
        reader = csv.reader(open(self.fullpath, 'rb'))
        return reader.next()
    
    def getPreviewBody(self):
        """Get the first 5 rows of attached CSV file.
        """
        result = []
        num = 0
        for row in self.reader:
            if num > 4:
                break
            num += 1
            row = row.items()
            # Sort fields in headerfield order
            row = sorted(row, key=lambda k: self.reader.fieldnames.index(k[0]))
            row = [x[1] for x in row]
            result.append(row)
        result.append(len(result[0]) * ['...'])
        return result

    def getPossibleHeaders(self):
        """Get the possible headers.

        The headers are described as dicts {value:internal_name,
        title:displayed_name}
        """
        result = [dict(title='<IGNORE COL>', value='--IGNORE--')]
        headers = self.importer.getHeaders()
        result.extend([dict(title=x, value=x) for x in headers])
        return result

    def getWarnings(self):
        import sys
        result = []
        try:
            self.importer.checkHeaders(self.headerfields, mode=self.mode)
        except:
            fatal = '%s' % sys.exc_info()[1]
            result.append(fatal)
        return result
    
    @property
    def nextstep(self):
        return self.url(self.context, '@@import4')

    def update(self, headerfield=None, back2=None, cancel=None, proceed=None):
        session = ISession(self.request)['waeup.sirp']
        self.filename = session.get('import_filename', None)
        self.mode = session.get('import_mode', None)
        self.importername = session.get('import_importer', None)
        
        if None in (self.filename, self.mode, self.importername):
            self.redirect(self.url(self.context, '@@import2'))
            return
        if back2 is not None:
            self.redirect(self.url(self.context ,'@@import2'))
            return
        if cancel is not None:
            self.flash('Import aborted.')
            self.redirect(self.url(self.context))
            return

        self.fullpath = os.path.join(self.context.storage, self.filename)
        self.headerfields = headerfield or self.getPreviewHeader()
        session['import_headerfields'] = self.headerfields

        if proceed is not None:
            self.redirect(self.url(self.context, '@@import4'))
            return
        
        self.importer = getUtility(IBatchProcessor, name=self.importername)
        self.reader = csv.DictReader(open(self.fullpath, 'rb'))

class DatacenterImportStep4(WAeUPPage):
    """Manual import step 4: do actual import
    """
    grok.context(IDataCenter)
    grok.name('import4')
    grok.template('datacenterimport4page')
    grok.require('waeup.manageUniversity')
    title = u'Process CSV file'
    pnav = 0

    filename = None
    mode = None
    importername = None
    headerfields = None
    warnnum = None

    def update(self, back=None, finish=None, showlog=None):
        if finish is not None:
            self.redirect(self.url(self.context))
            return
        if back is not None:
            self.redirect(self.url(self.context, '@@import3'))
            return
        session = ISession(self.request)['waeup.sirp']
        self.filename = session.get('import_filename', None)
        self.mode = session.get('import_mode', None)
        self.importername = session.get('import_importer', None)
        self.headerfields = session.get('import_headerfields', None)
        
        if None in (self.filename, self.mode, self.importername,
                    self.headerfields):
            self.redirect(self.url(self.context, '@@import3'))
            return

        if showlog is not None:
            logfilename = "datacenter.log"
            session['logname'] = logfilename
            self.redirect(self.url(self.context, '@@show'))
            return
            
        self.fullpath = os.path.join(self.context.storage, self.filename)
        self.importer = getUtility(IBatchProcessor, name=self.importername)

        # Perform batch processing...
        # XXX: This might be better placed in datacenter module.
        (linenum, self.warn_num,
         fin_path, pending_path) = self.importer.doImport(
            self.fullpath, self.headerfields, self.mode,
            self.request.principal.id, logger=self.context.logger)
        # Put result files in desired locations...
        self.context.distProcessedFiles(
            self.warn_num == 0, self.fullpath, fin_path, pending_path,
            self.mode)

        if self.warn_num:
            self.flash('Processing of %d rows failed!' % self.warn_num)
        self.flash('Successfully processed %s rows' % (
                linenum - (self.warn_num)))

class DatacenterLogsOverview(WAeUPPage):
    grok.context(IDataCenter)
    grok.name('logs')
    grok.template('datacenterlogspage')
    grok.require('waeup.manageUniversity')
    title = u'Data Center Logs'
    pnav = 0

    def update(self, show=None, logname=None, back=None):
        session = ISession(self.request)['waeup.sirp']
        if back is not None:
            self.redirect(self.url(self.context))
            return
        if logname is not None:
            session['logname'] = logname
        if show is not None:
            self.redirect(self.url(self.context, '@@show'))
            return
        self.files = self.context.getLogFiles()

class DatacenterLogsFileview(WAeUPPage):
    grok.context(IDataCenter)
    grok.name('show')
    grok.template('datacenterlogsshowfilepage')
    grok.require('waeup.manageUniversity')
    title = u'Show file'
    pnav = 0

    def update(self, show=None, remove=None, back=None):
        session = ISession(self.request)['waeup.sirp']
        logname = session.get('logname', None)
        if back is not None or logname is None:
            self.redirect(self.url(self.context, '@@logs'))
            return
        self.filename = logname
        self.files = self.context.getLogFiles()
        fullpath = os.path.join(self.context.storage, 'logs', logname)
        self.filecontents = open(fullpath, 'rb').read()

class DatacenterSettings(WAeUPPage):
    grok.context(IDataCenter)
    grok.name('manage')
    grok.template('datacentermanagepage')
    grok.require('waeup.manageUniversity')
    title = u'Data Center Settings'
    pnav = 0

    def update(self, newpath=None, move=False, overwrite=False,
               save=None, cancel=None):
        if move:
            move = True
        if overwrite:
            overwrite = True
        if newpath is None:
            return
        if cancel is not None:
            self.redirect(self.url(self.context))
            return
        try:
            not_copied = self.context.setStoragePath(newpath, move=move)
            for name in not_copied:
                self.flash('File already existed (not copied): %s' % name)
        except ValueError:
            self.flash('Given storage path cannot be used.')
            self.flash('Error: %s' %sys.exc_info()[1])
            return
        if newpath:
            self.flash('New storage path succefully set.')
            self.redirect(self.url(self.context))
        return

class ExportXMLPage(grok.View):
    """Deliver an XML representation of the context.
    """
    grok.name('export.xml')
    grok.require('waeup.manageUniversity')
    
    def render(self):
        exporter = IWAeUPXMLExporter(self.context)
        xml = exporter.export().read()
        self.response.setHeader(
            'Content-Type', 'text/xml; charset=UTF-8')
        return xml

class ImportXMLPage(WAeUPPage):
    """Replace the context object by an object created from an XML
       representation.

       XXX: This does not work for ISite objects, i.e. for instance
            for complete University objects.
    """
    grok.name('importxml')
    grok.require('waeup.manageUniversity')

    def update(self, xmlfile=None, CANCEL=None, SUBMIT=None):
        if CANCEL is not None:
            self.redirect(self.url(self.context))
            return
        if not xmlfile:
            return
        importer = IWAeUPXMLImporter(self.context)
        obj = importer.doImport(xmlfile)
        if type(obj) != type(self.context):
            return
        parent = self.context.__parent__
        name = self.context.__name__
        self.context = obj
        if hasattr(parent, name):
            setattr(parent, name, obj)
        else:
            del parent[name]
            parent[name] = obj
            pass
        return



#
# FacultyContainer pages...
#

class FacultyContainerPage(WAeUPPage):
    """ Index page for faculty containers.
    """
    grok.context(IFacultyContainer)
    grok.require('waeup.View')
    grok.name('index')
    title = 'Academics'
    pnav = 1

    def update(self, code=None, edit=None, delete=None):
        warning.need()
        if delete is not None and code is not None:
            try:
                del self.context[code]
                self.flash('Faculty %s successfully removed.' % code)                
            except:
                self.flash('Could not delete %s.' % code)

class AddFacultyFormPage(WAeUPAddFormPage):
    """ Page form to add a new faculty to a faculty container.
    """
    grok.context(IFacultyContainer)
    grok.require('waeup.manageUniversity')
    grok.name('add')
    form_fields = grok.AutoFields(IFaculty)
    label = ''
    title = 'Add faculty'
    pnav = 1

    @grok.action('Add faculty')
    def addFaculty(self, **data):
        faculty = createObject(u'waeup.Faculty')
        self.applyData(faculty, **data)
        try:
            self.context.addFaculty(faculty)
        except KeyError:
            self.status = Invalid('The name chosen already exists '
                                  'in the database')
            return
        except DuplicationError:
            self.status = Invalid('The name chosen already exists '
                                  'in the database')
            return
        self.redirect(self.url(self.context))
        
    @grok.action('Cancel')
    def cancel(self, **data):
        self.redirect(self.url(self.context))   

class RemoveFacultyFormPage(RemoveFormPage):
    """ Remove Faculty Page 
    """
    grok.context(IFacultyContainer)
    title = "Remove faculty"
    pnav = 1

#
# Faculty pages
#
class FacultyPage(WAeUPPage):
    """Index page of faculties.
    """
    grok.context(IFaculty)
    grok.require('waeup.View')
    grok.name('index')
    pnav = 1
    @property
    def title(self):
        return "%s (%s)" % (getName(self.context), self.context.code)

    def update(self, code=None, edit=None, delete=None):
        warning.need()
        if delete is not None and code is not None:
            try:
                del self.context[code]
                self.flash('Department %s successfully removed.' % code)                
            except:
                self.flash('Could not delete %s.' % code)

class AddDepartmentFormPage(WAeUPAddFormPage):
    """Add a department to a faculty.
    """
    grok.context(IFaculty)
    grok.name('add')
    grok.require('waeup.manageUniversity')
    form_fields = grok.AutoFields(IDepartment)
    title = 'Add department'
    label = ''
    pnav = 1
    
    @grok.action('Add department')
    def addDepartment(self, **data):
        department = createObject(u'waeup.Department')
        self.applyData(department, **data)
        try:
            self.context.addDepartment(department)
        except KeyError:
            self.status = Invalid('The code chosen already exists '
                                  'in the database')
            return
        except DuplicationError:
            self.status = Invalid('The code chosen already exists '
                                  'in the database')
            return
        self.redirect(self.url(self.context))
        
    @grok.action('Cancel')
    def cancel(self, **data):
        self.redirect(self.url(self.context))           

class ManageFacultyFormPage(WAeUPEditFormPage):
    """Manage the basic properties of a `Faculty` instance.
    """
    grok.context(IFaculty)
    grok.name('manage')
    grok.require('waeup.manageUniversity')
    form_fields = grok.AutoFields(IFaculty)
    title = 'Edit faculty'
    pnav = 1

    @grok.action('Save')
    def save(self, **data):
        self.applyData(self.context, **data)
        return
    
    @grok.action('Save and return')
    def saveAndReturn(self, **data):
        self.applyData(self.context, **data)
        self.redirect(self.url(self.context))
        return

    @grok.action('Cancel', validator=NullValidator)
    def cancel(self, **data):
        self.redirect(self.url(self.context))
        return


class RemoveDepartmentFormPage(RemoveFormPage):
    """ Remove Department Page 
    """
    grok.context(IFaculty)
    title = "Remove department"
    pnav = 1


#
# Department pages
#
class DepartmentPage(WAeUPPage):
    """Department index page.
    """
    grok.context(IDepartment)
    grok.require('waeup.View')
    grok.name('index')
    pnav = 1

    @property
    def title(self):
        return "%s (%s)" % (getName(self.context), self.context.code)
    
    def update(self):
        yui.tabview.need()

        if 'delcourse' in self.request.form:
            code = self.request.form['code']
            del self.context.courses[code]
        if 'delcert' in self.request.form:
            code = self.request.form['code']
            del self.context.certificates[code]
        self.tables = ITableProvider(self.context).getTables(view=self)
        self.coursetable = self.tables[0]
        self.certificatetable = self.tables[1]
        self.coursetable.need()

class DepartmentManageFormPage(WAeUPEditFormPage):
    """Manage the basic properties of a `Department` instance.
    """
    grok.context(IDepartment)
    grok.name('manage')
    grok.require('waeup.manageUniversity')
    title = u'Edit department'
    pnav = 1

    form_fields = grok.AutoFields(IDepartment)

    @grok.action('Save')
    def save(self, **data):
        self.applyData(self.context, **data)
        return
    
    @grok.action('Save and return')
    def saveAndReturn(self, **data):
        self.applyData(self.context, **data)
        self.redirect(self.url(self.context))
        return

    @grok.action('Cancel', validator=NullValidator)
    def cancel(self, **data):
        self.redirect(self.url(self.context))
        return

class DepartmentAddCourseFormPage(WAeUPAddFormPage):
    """Add-form to add course to a department.
    """
    grok.context(IDepartment)
    grok.name('addcourse')
    grok.require('waeup.manageUniversity')
    title = u'Add course'
    pnav = 1
    
    # We need a deepcopy here, because otherwise also all other
    # ICourse forms would be affected.
    form_fields = copy.deepcopy(grok.AutoFields(ICourse))
    form_fields.get('code').field.readonly = False

    @grok.action('Add course')
    def addCourse(self, **data):
        course = createObject(u'waeup.Course')
        self.applyData(course, **data)
        try:
            self.context.courses.addCourse(course)
        except KeyError:
            self.status = Invalid('The code chosen already exists '
                                  'in the database')
            return
        except DuplicationError:
            self.status = Invalid('The code chosen already exists '
                                  'in the database')
            return
        self.redirect(self.url(self.context))

    @grok.action('Cancel', validator=NullValidator)
    def cancel(self, **data):
        self.redirect(self.url(self.context))
        return

class DepartmentAddCertificateFormPage(WAeUPAddFormPage):
    """Add-form to add certificate to a department.
    """
    grok.context(IDepartment)
    grok.name('addcertificate')
    grok.require('waeup.manageUniversity')
    title = u'Add certificate'
    pnav = 1
    form_fields = grok.AutoFields(ICertificate)
    
    @grok.action('Add certificate')
    def addCertificate(self, **data):
        certificate = createObject(u'waeup.Certificate')
        try:
            self.applyData(certificate, **data)
        except NoTransitionAvailableError:
            self.status = Invalid('Review state cannot be changed to '
                                  'requested state: no such transition '
                                  'available in workflow.')
            return
        try:
            self.context.certificates.addCertificate(certificate)
        except KeyError:
            self.status = Invalid('The name chosen already exists '
                                  'in the database')
            return
        except DuplicationError:
            self.status = Invalid('The name chosen already exists '
                                  'in the database')
            return
        self.redirect(self.url(self.context))

    @grok.action('Cancel', validator=NullValidator)
    def cancel(self): #, **data):
        self.redirect(self.url(self.context))
        return

#
# Courses pages
#
class CoursePage(WAeUPDisplayFormPage):
    """Course index page.
    """
    grok.context(ICourse)
    grok.name('index')
    grok.require('waeup.View')
    pnav = 1
    form_fields = grok.AutoFields(ICourse)

    @property
    def title(self):
        return 'Course: %s (%s)' % (self.context.title, self.context.code)

class CourseManageFormPage(WAeUPEditFormPage):
    """Edit form page for courses.
    """
    grok.context(ICourse)
    grok.name('manage')
    grok.require('waeup.manageUniversity')
    title = u'Edit course'
    pnav = 1

    form_fields = grok.AutoFields(ICourse)

    @grok.action('Save')
    def save(self, **data):
        self.applyData(self.context, **data)
        return
    
    @grok.action('Save and return')
    def saveAndReturn(self, **data):
        self.applyData(self.context, **data)
        self.redirect(self.url(self.context))
        return

    @grok.action('Cancel', validator=NullValidator)
    def cancel(self, **data):
        self.redirect(self.url(self.context))
        return

#
# Certificate pages
#
class CertificatePage(WAeUPPage):
    """Index page for certificates.
    """
    grok.context(ICertificate)
    grok.name('index')
    grok.require('waeup.View')
    pnav = 1

    @property
    def title(self):
        return "%s (%s)" % (self.context.title, self.context.code)

    def update(self):
        if 'delcourse' in self.request.form:
            code = self.request.form['code']
            level = self.request.form['level']
            self.context.delCourseRef(code, level=level)
        return
        
    def getCourseList(self):
        def cmpCourses(a, b):
            if a['level'] < b['level']:
                return -1
            if a['level'] > b['level']:
                return 1
            if a['mandatory'] != b['mandatory']:
                if a['mandatory'] == False and b['mandatory'] == True:
                    return -1
                return 1
            if a['code'] < b['code']:
                return -1
            if a['code'] > b['code']:
                return 1
            return 0

        entries = []
        levels = dict()
        for key, course in self.context.items():
            certcourse = self.context[key]
            course = certcourse.course
            level = certcourse.level
            mandatory = certcourse.core_or_elective
            title = course.title
            code = course.code
            course_url = self.url(self.context[key])
            semester = course.semester

            new_entry = dict(
                level=level, mandatory=mandatory, url=course_url,
                title=title, code=code, semester=semester)
            if str(level) not in levels.keys():
                levels[str(level)] = dict()
            if str(semester) not in levels[str(level)].keys():
                levels[str(level)][str(semester)] = []
            levels[str(level)][str(semester)].append(new_entry)
        return levels

class CertificateManageForm(WAeUPEditFormPage):
    """Manage the basic properties of a `Certificate` instance.
    """
    grok.context(ICertificate)
    grok.name('manage')
    grok.require('waeup.manageUniversity')
    # We omit 'code' field from original schema and set prepend a new
    # one with 'readonly' set to True.
    form_fields = grok.Fields(
        code = schema.TextLine(
            title = u'Code', readonly=True
            )
        ) + grok.AutoFields(ICertificate).omit('code')
    pnav = 1

    @property
    def title(self):
        # Set certificate name as form label
        return self.form_fields['title'].field.get(self.context)
    
    @grok.action('Save')
    def save(self, **data):
        try:
            self.applyData(self.context, **data)
        except NoTransitionAvailableError:
            self.status = Invalid('Review state cannot be changed to '
                                  'requested state: no such transition '
                                  'available in workflow.')
        return
    
    @grok.action('Save and return')
    def saveAndReturn(self, **data):
        try:
            self.applyData(self.context, **data)
        except NoTransitionAvailableError:
            self.status = Invalid('Review state cannot be changed to '
                                  'requested state: no such transition '
                                  'available in workflow.')
            return
        self.redirect(self.url(self.context))
        return

    @grok.action('Cancel', validator=NullValidator)
    def cancel(self, **data):
        self.redirect(self.url(self.context))
        return

#
# Certificate course pages...
#
class CertificateCoursePage(WAeUPDisplayFormPage):
    """Course index page.
    """
    grok.context(ICertificateCourse)
    grok.name('index')
    grok.require('waeup.View')
    pnav = 1
    form_fields = grok.AutoFields(ICertificateCourse)

    @property
    def title(self):
        return self.context.course.title

class CertificateCourseFormPage(WAeUPEditFormPage):
    """Manage the basic properties of a `CertificateCourse` instance.
    """
    grok.context(ICertificateCourse)
    grok.name('manage')
    grok.require('waeup.manageUniversity')
    form_fields = grok.AutoFields(ICertificateCourse)
    pnav = 1
    
    @property
    def title(self):
        return 'Edit certificate course %s' % self.context.course.code
    
    @grok.action('Save and return')
    def saveAndReturn(self, **data):
        parent = self.context.__parent__
        if self.context.level == data['level']:
            self.applyData(self.context, **data)
        else:
            # The level changed. We have to create a new certcourse as
            # the key of the entry will change as well...
            old_level = self.context.level
            parent.addCourseRef(**data)
            parent.delCourseRef(data['course'].code, level=old_level)
        self.redirect(self.url(parent))
        return

    @grok.action('Cancel', validator=NullValidator)
    def cancel(self, **data):
        self.redirect(self.url(self.context))
        return

class CertificateCourseAddFormPage(WAeUPAddFormPage):
    """Add-page to add a course ref to a certificate
    """
    grok.context(ICertificate)
    grok.name('addcertificatecourse')
    grok.require('waeup.manageUniversity')
    form_fields = grok.AutoFields(ICertificateCourse)
    pnav = 1
    title = 'Add course'

    @grok.action('Add course')
    def addCertcourse(self, **data):
        try:
            self.context.addCourseRef(**data)
        except KeyError:
            self.status = Invalid('The chosen course is already part of'
                                  'this certificate')
            return
        except DuplicationError:
            self.status = Invalid('The chosen course is already part of'
                                  'this certificate')
            return
        self.redirect(self.url(self.context))

    @grok.action('Cancel', validator=NullValidator)
    def cancel(self, **data):
        self.redirect(self.url(self.context))
        return
